package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;

public class CheckStringInContact extends AbstractBoundaryCheck<StringBoundary> {

  private static CheckStringInContact instance = new CheckStringInContact();

  private CheckStringInContact() {
    super();
  }

  public static CheckStringInContact getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_IN_CONTACT;
  }

  @Override
  public boolean check(StringBoundary b1, StringBoundary b2) {
    return checkInContact(b1, b2);
  }

  public static boolean checkInContact(StringBoundary b1, StringBoundary b2) {
    if (b1.matchesAny()
        || b2.matchesAny()
        || b1.matchesNoneOfValues() && b2.matchesNoneOfValues()) {
      return true;
    }
    int[] b1h = b1.getValuesHashes();
    int[] b2h = b2.getValuesHashes();
    // both are not matching non values
    if (!b1.matchesNoneOfValues() && !b2.matchesNoneOfValues()) {
      for (int i : b1h) {
        for (int u : b2h) {
          if (i == u) {
            return true;
          }
        }
      }
      return false;
    }
    // one is matching non values "not("a")"
    if (b1.matchesNoneOfValues()) {
      outer:
      for (int i : b2h) {
        for (int u : b1h) {
          if (i == u) {
            continue outer;
          }
        }
        return true;
      }
      return false;
    }
    if (b2.matchesNoneOfValues()) {
      outer:
      for (int i : b1h) {
        for (int u : b2h) {
          if (i == u) {
            continue outer;
          }
        }
        return true;
      }
      return false;
    }
    return false;
  }
}
